#include "kanji_union.h"

NORMAL_SUBSTITUTE data[DATA_ARRAY_LIMIT];

KANJI kanji_union;
KANJI *double_byte_character = &kanji_union;

int inverted(char *c) {
    char *p = (char *)double_byte_character->sequence;

// Copying in reverse order
    p[0] = c[1];
    p[1] = c[0];
    return double_byte_character->code[0];
}

int kanjiCode(std::string & line) {
    using namespace std;

    bool kanji = false;
    char *buffer;
    char *p;
    int before;
    int k, n;
    int size = line.length();
    int size_1 = size + 1;
    unsigned char uc;

    try {
        buffer = new char[size_1];
        strcpy(buffer, line.c_str() );

        for(int i = 0; i < size; ++i) {
            uc = (unsigned char)buffer[i];
            k = (int)uc;

            if(kanji) {
                p = & buffer[before];
                n = inverted(p);
                break;
            }

            if(0x81 <= k && k <= 0xFC) {
                if (KANA_KUTEN <= k && k <= KANA_HANDAKUTEN) {
                    n = k;
                    break;
                } else {
                    before = i;
                    kanji = true;
                }

            } else {
                n = k;
                break;
            }

        }

        delete[] buffer;
    } catch(...) {
        cerr << "new char[] error-7: " << size_1 << endl;
        exit(1);
    }

    return n;
}

bool editRecord(std::string & normal, std::string & substitute, std::string & binaryName, bool no_searching, bool *whichTable) {
    using namespace std;

    char buffer[BLOCK_SIZE];
    char c[DOUBLE_KANJI];
    int buffer_size = BLOCK_SIZE;
    int from, into, k;
    OPEN_CLOSE_FUNCTION *openClose = new OPEN_CLOSE_FUNCTION[SEARCH_COUNT];
    stringstream os;
    unsigned char uc;

    openClose[0] = ms932_sjis_file_in_installed_directory;
    openClose[1] = ms932_sjis_file_in_current_directory;
    openClose[2] = ms932_sjis_file_in_path_directory;
    openClose[3] = ms932_sjis_file_in_specified_directory;

    for(int i = 0; i < SEARCH_COUNT; ++i) {
        if( openClose[i](buffer, buffer_size) ) {
            if(*whichTable) {
                cout << buffer << " is primarily accessible to dayu." << endl;
                *whichTable = false;
            }

            break;
        } else {
            switch(i) {
            case 0:
                if(no_searching) {
                    cerr << "openClose[" << i << "]() error-1: " << MS932_SJIS_BIN << endl;
                    delete[] openClose;
                    return false;
                }

                break;
            case 3:
                cerr << "openClose[" << i << "]() error-2: " << MS932_SJIS_BIN << endl;
                delete[] openClose;
                return false;
            default:
                break;
            }

        }

    }

    SparseSJIS *sjis = new SparseSJIS();
    sjis->clear();
    if(! sjis->setting(no_searching, whichTable) ) {
        errorHandler("editRecord() error:", "sjis->setting(no_searching, whichTable)");
        delete sjis;
        delete[] openClose;
        return false;
    }

    from = kanjiCode(normal);
    cout << hex << from << RIGHT_ARROW;

    into = kanjiCode(substitute);
    cout << hex << into << endl;

    ifstream binaryFile(buffer);
    for(int i = 1; i <= RECORD_COUNT; ++i) {
        binaryFile.read(c, DOUBLE_KANJI);
        if(i < FIRST_KANJI) {
            uc = (unsigned char)c[0];
            k = (int)uc;
        } else {
            k = inverted(c);
        }

        os << c[0] << c[1];

        if(k == from) {
            os << sjis->working(into, 0) << sjis->working(into, 1);
        } else {
            os << c[SJIS_KANJI_SIZE] << c[ SJIS_KANJI_SIZE + 1 ];
        }

    }

    binaryFile.close();
    ofstream editedFile( binaryName.c_str(), ios::out | ios::binary | ios::trunc );
    editedFile << os.str();
    delete sjis;
    delete[] openClose;
    return true;
}

bool editManyRecords(std::string & text, std::string & binaryName, bool no_searching, bool *whichTable) {
    using namespace std;

    bool checked = false;
    char buffer[BLOCK_SIZE];
    char c[DOUBLE_KANJI];
    int buffer_size = BLOCK_SIZE;
    int from, into, k;
    int m = 0;
    int size = 0;
    int used = 0;
    OPEN_CLOSE_FUNCTION *openClose = new OPEN_CLOSE_FUNCTION[SEARCH_COUNT];
    stringstream os;
    unsigned char uc;

    openClose[0] = ms932_sjis_file_in_installed_directory;
    openClose[1] = ms932_sjis_file_in_current_directory;
    openClose[2] = ms932_sjis_file_in_path_directory;
    openClose[3] = ms932_sjis_file_in_specified_directory;

    for(int i = 0; i < SEARCH_COUNT; ++i) {
        if( openClose[i](buffer, buffer_size) ) {
            if(*whichTable) {
                cout << buffer << " is primarily accessible to dayu." << endl;
                *whichTable = false;
            }

            break;
        } else {
            switch(i) {
            case 0:
                if(no_searching) {
                    cerr << "openClose[" << i << "]() error-3: " << MS932_SJIS_BIN << endl;
                    delete[] openClose;
                    return false;
                }

                break;
            case 3:
                cerr << "openClose[" << i << "]() error-4: " << MS932_SJIS_BIN << endl;
                delete[] openClose;
                return false;
            default:
                break;
            }

        }

    }

    SparseSJIS *sjis = new SparseSJIS();
    sjis->clear();
    if(! sjis->setting(no_searching, whichTable) ) {
        errorHandler("editManyRecords() error:", "sjis->setting(no_searching, whichTable)");
        delete sjis;
        delete[] openClose;
        return false;
    }

    ifstream sjisFile( text.c_str() );
    if( sjisFile.fail() ) {
        cerr << "ifstream sjisFile() error: " << text << endl;
        delete sjis;
        delete[] openClose;
        return false;
    }

    char record[BLOCK_SIZE];
    string *d = new string("");
    while( sjisFile.getline(record, BLOCK_SIZE) ) {
        delete d;
        d = new string(record);
        size = d->length();
        if(size == DUAL_KANJI) {
            data[m].please_check = true;
            data[m].normal = d->substr(0, 2);
            data[m].substitute = d->substr(SJIS_KANJI_SIZE, 2);
        } else {
            data[m].please_check = true;
            data[m].normal = d->substr(0, 1); // katakana, for example 
            data[m].substitute = d->substr(KATAKANA_SIZE, 1);
        }

        used = m;
        m += 1;
        if(m >= DATA_ARRAY_LIMIT) {
            break;
        }

    }

    ifstream binaryFile(buffer);
    for(int i = 1; i <= RECORD_COUNT; ++i) {
        binaryFile.read(c, DOUBLE_KANJI);
        if(i < FIRST_KANJI) {
            uc = (unsigned char)c[0];
            k = (int)uc;
        } else {
            k = inverted(c);
        }

        os << c[0] << c[1];

        for(int m = 0; m <= used; ++m) {
            if(data[m].please_check) {
                from = kanjiCode(data[m].normal);
                if(k == from) {
                    into = kanjiCode(data[m].substitute);
                    checked = true;
                    data[m].please_check = false;
                    break;
                } else {
                    checked = false;
                }

            } else {
                checked = false;
            }

        }

        if(checked) {
            os << sjis->working(into, 0) << sjis->working(into, 1);
        } else {
            os << c[SJIS_KANJI_SIZE] << c[ SJIS_KANJI_SIZE + 1 ];
        }

    }

    binaryFile.close();
    ofstream editedFile( binaryName.c_str(), ios::out | ios::binary | ios::trunc );
    editedFile << os.str();
    delete d;
    delete sjis;
    delete[] openClose;
    return true;
}

std::string errorHandler(const char *p, char *q) {
    using namespace std;

    const string GAP(" ");
    stringstream os;
    os << p << GAP << q;
    string messages = os.str();

    ofstream errorFile(ERROR_TEXT);
    errorFile << messages;
    return messages;
}

std::string errorHandler(const char *p, const char *q) {
    using namespace std;

    const string GAP(" ");
    stringstream os;
    os << p << GAP << q;
    string messages = os.str();

    ofstream errorFile(ERROR_TEXT);
    errorFile << messages;
    return messages;
}
