#include "kanji_union.h"

extern char *binary_name;

bool ms932_sjis_file_in_current_directory(char *buffer, int buffer_size) {
    using namespace std;

    bool b = true;
    ifstream in(MS932_SJIS_BIN); // current directory
    if( in.fail() ) {
        b = false;
        memset(buffer, 0, buffer_size);
    } else {
        strcpy(buffer, MS932_SJIS_BIN);
    }

    return b;
}

bool ms932_sjis_file_in_installed_directory(char *buffer, int buffer_size) {
    using namespace std;

    bool b = true;
    ifstream in(C_MS932_SJIS_BIN); // directory where dayu was installed.
    if( in.fail() ) {
        b = false;
        memset(buffer, 0, buffer_size);
    } else {
        strcpy(buffer, C_MS932_SJIS_BIN);
    }

    return b;
}

bool ms932_sjis_file_in_path_directory(char *buffer, int buffer_size) {
    using namespace std;

    bool b = true;
    CommandPath *c = new CommandPath();
    string included = c->pathName(MS932_SJIS_BIN); // directory included in Path

    if( included.length() == 0 ) {
        b = false;
        memset(buffer, 0, buffer_size);
    } else {
        strcpy(buffer, included.c_str() );
    }

    delete c;
    return b;
}

bool ms932_sjis_file_in_specified_directory(char *buffer, int buffer_size) {
    using namespace std;

    bool b = true;
    ifstream in(binary_name);
    if( in.fail() ) {
        b = false;
        memset(buffer, 0, buffer_size);
    } else {
        strcpy(buffer, binary_name);
    }

    return b;
}

bool SparseSJIS::setting(bool no_searching, bool *whichTable) {
    using namespace std;

    char buffer[BLOCK_SIZE];
    char c[DOUBLE_KANJI];
    int buffer_size = BLOCK_SIZE;
    int index;
    OPEN_CLOSE_FUNCTION *openClose = new OPEN_CLOSE_FUNCTION[SEARCH_COUNT];
    unsigned char uc;

    openClose[0] = ms932_sjis_file_in_installed_directory;
    openClose[1] = ms932_sjis_file_in_current_directory;
    openClose[2] = ms932_sjis_file_in_path_directory;
    openClose[3] = ms932_sjis_file_in_specified_directory;

    for(int i = 0; i < SEARCH_COUNT; ++i) {
        if( openClose[i](buffer, buffer_size) ) {
            if(*whichTable) {
                cout << buffer << " is primarily accessible to dayu." << endl;
                *whichTable = false;
            }

            break;
        } else {
            switch(i) {
            case 0:
                if(no_searching) {
                    cerr << "openClose[" << i << "]() error-5: " << MS932_SJIS_BIN << endl;
                    delete[] openClose;
                    return false;
                }

                break;
            case 3:
                cerr << "openClose[" << i << "]() error-6: " << MS932_SJIS_BIN << endl;
                delete[] openClose;
                return false;
            default:
                break;
            }

        }

    }

    ifstream binaryFile(buffer);
    for(int i = 1; i <= RECORD_COUNT; ++i) {
        binaryFile.read(c, DOUBLE_KANJI);
        if(i < FIRST_KANJI) {
            uc = (unsigned char)c[0];
            index = (int)uc;
            populated(index, 0, c[SJIS_KANJI_SIZE]);
        } else {
            populated( inverted(c), 0, c[SJIS_KANJI_SIZE]);
            populated( inverted(c), 1, c[ SJIS_KANJI_SIZE + 1 ]);
        }

    }

    delete[] openClose;
    return true;
}

std::string SparseSJIS::working(int index) {
    using namespace std;

    int k = index * SJIS_KANJI_SIZE_1;
    char *p = sparse + k;
    stringstream os;
    os << p;
    return os.str();
}

unsigned char SparseSJIS::working(int index, int place) {
    int k = index * SJIS_KANJI_SIZE_1 + place;
    char* p = sparse + k;
    unsigned char c = (unsigned char) *p;
    return c;
}
