#ifndef KANJI_UNION_H
#define KANJI_UNION_H

#ifdef __BORLANDC__
#pragma warn -8004
#pragma warn -8012
#endif

#define VERSION_NUMBER 1.215
#define BELL 0x07
#define BELL_ASCII_KATAKANA_COUNT 224
#define BLOCK_SIZE 2560
#define BUFFER_SIZE 524288 // 512 KB
#define C_MS932_SJIS_BIN "C:\\sjisEUCjpCxx\\MS932~Shift_JIS.bin"
#define C_SIZE 256 // FILE_NAME_SIZE + 1
#define CARRIAGE_RETURN_LINE_FEED "-CRLF"
#define DATA_ARRAY_LIMIT 906
#define DO_NOT_REMOVE_TEMPORARY_BINARY_FILE "-y"
#define DO_NOT_SEARCH_TABLE "-d"
#define DOUBLE_KANJI 4 // SJIS_KANJI_SIZE + SJIS_KANJI_SIZE
#define DUAL_KANJI 4 // SJIS_KANJI_SIZE + SJIS_KANJI_SIZE
#define ERROR_TEXT "error.txt"
#define FILE_ERROR_CODE -2
#define FILE_NAME_SIZE 255
#define FIRST_KANJI 159
#define IDENTICAL_TIME "-t"
#define KANA_HANDAKUTEN 0xDF
#define KANA_KUTEN 0xA1
#define KANJI_SIZE 2
#define KATAKANA_SIZE 1
#define LINE_FEED "-LF"
#define MS932_SJIS_BIN "MS932~Shift_JIS.bin"
#define MS932_SJIS_TEXT "dayu.txt"
#define NO_EDITING "-e-"
#define NO_READING "-r-"
#define NORMAL_ARRAY_SIZE 3
#define RECORD_COUNT 7882
#define RIGHT_ARROW "  " // Shift_JIS
#define SEARCH_COUNT 4
#define SJIS_KANJI_SIZE 2
#define SJIS_KANJI_SIZE_1 3
#define SKIPPING "-i"
#define SUBSTITUTE_ARRAY_SIZE 3
#define TEMPORARY_BINARY "binary.tmp"
#define TEMPORARY_BINARY_FILE "-b"
#define WHICH_TABLE "-w"
#define ZERO "0"
#include <stdio.h>
#include <stdlib.h> // getenv()
#include "Dividing.h"

typedef bool (*OPEN_CLOSE_FUNCTION)(char *buffer, int buffer_size);

typedef union KANJI KANJI;
union KANJI {
    unsigned char sequence[6];
    unsigned short code[3];
    unsigned int block;
};

typedef struct NORMAL_SUBSTITUTE NORMAL_SUBSTITUTE;
struct NORMAL_SUBSTITUTE {
    bool please_check;
    std::string normal;
    std::string substitute;
};

class CommandPath {
    char *data;

public:
    CommandPath() {
        data = new char[BUFFER_SIZE];
        strcpy(data, getenv("Path") );
    }

    ~CommandPath() {
        delete[] data;
    }

    char *path_name(char *extra);
    std::string pathName(const char *extra);
};

class Regulate {
    const int ERROR_CODE;
    bool carriage_return, no_searching, whichTable;
    char *buffer;
    std::string documentName;

public:
    Regulate(char *d, bool c, bool n, bool w): ERROR_CODE(1) {
        std::string document(d);
        documentName = document;
        carriage_return = c;
        no_searching = n;
        whichTable = w;
        buffer = new char[BUFFER_SIZE];
    }

    Regulate(std::string & d, bool c, bool n, bool w): ERROR_CODE(1) {
        documentName = d;
        carriage_return = c;
        no_searching = n;
        whichTable = w;
        buffer = new char[BUFFER_SIZE];
    }

    ~Regulate() {
        delete[] buffer;
    }

    int readingToProduceSJIS(char *new_name);
    int skippingControlCharactersToProduceSJIS(char *new_name);
};

class Standard_input {
    const int ERROR_CODE;
    bool no_searching, whichTable;
    char *buffer;

public:
    Standard_input(bool n, bool w): ERROR_CODE(1) {
        no_searching = n;
        whichTable = w;
        buffer = new char[BUFFER_SIZE];
    }

    ~Standard_input() {
        delete[] buffer;
    }

    int standard_input_output();
    int standard_input_output_skipping();
};

class SparseSJIS {
    char *sparse;
    int size;

public:
    SparseSJIS() {
        size = 0xFC4C * SJIS_KANJI_SIZE_1;
        sparse = new char[size];
    }

    ~SparseSJIS() {
        delete[] sparse;
    }

    void clear() {
        memset(sparse, 0, size);
    }

    void populated(int index, int place, char c) {
        int k = index * SJIS_KANJI_SIZE_1 + place;
        *(sparse + k) = c;
    }

    bool setting(bool no_searching, bool *whichTable);
    std::string working(int index);
    unsigned char working(int index, int place);
};

bool editManyRecords(std::string & text, std::string & binary_name, bool no_searching, bool *which_table);
bool editRecord(std::string & normal, std::string & substitute, std::string & binary_name, bool no_searching, bool *which_table);
bool ms932_sjis_file_in_current_directory(char *buffer, int buffer_size);
bool ms932_sjis_file_in_installed_directory(char *buffer, int buffer_size);
bool ms932_sjis_file_in_path_directory(char *buffer, int buffer_size);
bool ms932_sjis_file_in_specified_directory(char *buffer, int buffer_size);
bool openClose(char *file_name);
bool openClose(char *file_name, bool write);
bool openClose(std::string & fileName);
int inverted(char *c);
int kanjiCode(std::string & line);
int produceBinaryFile(const char *name, bool no_removing);
int renameBinaryFile(const char *name, bool no_removing);
std::string errorHandler(const char *p, char *q);
std::string errorHandler(const char *p, const char *q);
void acquire_parameters_from_text(const char *parameters_text, char *binary_name, char *document_name, char *new_name, char *normal, char *substitute, char *text_of_substitutes_list, bool *produce_binary_file, bool *carriage_return, bool *no_searching, bool *edit_table, bool *skip_control_characters, bool *no_reading, bool *identical_time, bool *which_table, bool *no_removing);
void versionGuide(double v, bool comments);

#endif // KANJI_UNION_H
