#include "hanzen.h"

extern SparseKana *kana;

namespace {
    bool LISTED = true;
    const bool NO_REMOVING = false;
    const int ERROR_CODE = -1;
    const int FIELD_COUNT_0 = 3;
    const int NEW_NAME_ERROR = 2;
    const int SEPARATOR_0 = ',';
    const long BUFFER_SIZE_0 = 6;
    char BITS_0[] = "111";
    char GAP[] = " ";
}

DestroyRebuild::DestroyRebuild(char *e, char *b, char *n, long z, int s, int f, bool r, bool w) {
    using namespace std;

    int phase = BUFFER;

    strcpy(file_name, e);
    strcpy(new_name, n);
    buffer_size = (int)z;
    field_count = f;
    remove_separators = r;
    separator = s;
    way_of_arranging = w;

    pattern[FIELD_COUNT] = '\0';
    strncpy(pattern, b, FIELD_COUNT);

    char *p = strchr(pattern, CLEAR);
    full_house = (p == NULL) ? true : false;

    try {
        buffer = new char[buffer_size];

        phase = COMBO;
        combo = new char[ MINIMUM_COMBO_SIZE + buffer_size ];
    } catch(...) {
        cerr << "new char[] error-7: " << buffer_size << " bytes ( phase = " << phase << " )" << endl;
        exit(1);
    }

}

HankakuZenkaku::HankakuZenkaku(char *e, char *b, char *n, long z, int s, int f, bool r, bool w, bool a, bool c, bool g, bool i, bool *which): DestroyRebuild(e, b, n, z, s, f, r, w) {
    using namespace std;

    way_of_arranging = false;

    abolish_hankaku_katakana = a;
    combining_character = c;
    no_searching = g;
    skip_control_characters = i;
    which_table = which;
}

int HankakuZenkaku::selecting(int field_number) {
    using namespace std;

    char *p;
    char few_fields;
    Dividing *d = new Dividing(BITS_0, BUFFER_SIZE_0, SEPARATOR_0, FIELD_COUNT_0, NO_REMOVING);
    EDIT_FUNCTION removing_katakana = hankaku_zenkaku;
    FILE *file;
    int exit_code = 0;
    long z = (long)buffer_size;
    ofstream new_file(new_name);
    if( new_file.fail() ) {
        exit_code = NEW_NAME_ERROR;
        errorHandler("ofstream new_file() error:", new_name);
        exit(1);
    }

    file = fopen(file_name, "rt");
    if(file == NULL) {
        cerr << "fopen() error-3: " << file_name  << endl;
        return ERROR_CODE;
    }

    kana = new SparseKana(buffer_size);
    if(! kana->setting(no_searching, which_table) ) {
        errorHandler("selecting() error:", "kana->setting()");
        delete d; delete kana;
        return ERROR_CODE;
    }

    if(skip_control_characters) {
        if(abolish_hankaku_katakana) {
            removing_katakana = skip_control_characters_to_abolish_hankaku;
        } else {
            if(combining_character) {
                removing_katakana = skip_control_characters_to_replace_hankaku_with_zenkaku;
            } else {
                removing_katakana = skip_control_characters_to_replace_hankaku_with_zenkaku_except_dakuten;
            }

        }

    } else {
        if(abolish_hankaku_katakana) {
            removing_katakana = abolish_hankaku;
        } else {
            if(combining_character) {
//                removing_katakana = hankaku_zenkaku;
            } else {
                removing_katakana = replace_hankaku_with_zenkaku_except_dakuten;
            }

        }

    }

    if(field_count <= 0x00FF) { // 0 to 255
        few_fields = (char)field_count;

        for(;;) {
            p = fgets(buffer, buffer_size, file);
            if(p == NULL) {
                break;
            }

            delete d;
            d = new Dividing(z, separator, few_fields, remove_separators);
            d->editing_fields_list(buffer, removing_katakana, field_number);
            new_file << d->unite(buffer, LISTED) << endl;
        }

    } else {
        for(;;) {
            p = fgets(buffer, buffer_size, file);
            if(p == NULL) {
                break;
            }

            delete d;
            d = new Dividing(z, separator, field_count, remove_separators);
            d->editing_fields_list(buffer, removing_katakana, field_number);
            new_file << d->unite(buffer, LISTED) << endl;
        }

    }

    fclose(file);
    delete d;
    return exit_code;
}

int HankakuZenkaku::selecting(int field_number, char binary_extra) {
    using namespace std;

    char *p;
    char few_fields;
    char extra[2] = {0, 0};
    Dividing *d = new Dividing(BITS_0, BUFFER_SIZE_0, SEPARATOR_0, FIELD_COUNT_0, NO_REMOVING);
    EDIT_FUNCTION removing_katakana = hankaku_zenkaku;
    FILE *file;
    int exit_code = 0;
    long z = (long)buffer_size;
    stringstream *combo = new stringstream();
    ofstream new_file(TEMPORARY_BINARY, ios::out | ios::binary | ios::trunc );
    if( new_file.fail() ) {
        exit_code = NEW_NAME_ERROR;
        errorHandler("ofstream new_file() error:", TEMPORARY_BINARY);
        exit(1);
    }

    file = fopen(file_name, "rt");
    if(file == NULL) {
        cerr << "fopen() error-4: " << file_name  << endl;
        delete d; delete combo;
        return ERROR_CODE;
    }

    kana = new SparseKana(buffer_size);
    if(! kana->setting(no_searching, which_table) ) {
        errorHandler("selecting() error:", "kana->setting()");
        delete d; delete kana; delete combo;
        return ERROR_CODE;
    }

    extra[0] = binary_extra;

    if(abolish_hankaku_katakana) {
        removing_katakana = abolish_hankaku;
    } else {
        if(combining_character) {
//            removing_katakana = hankaku_zenkaku;
        } else {
            removing_katakana = replace_hankaku_with_zenkaku_except_dakuten;
        }

    }

    if(field_count <= 0x00FF) { // 0 to 255
        few_fields = (char)field_count;

        for(;;) {
            p = fgets(buffer, buffer_size, file);
            if(p == NULL) {
                break;
            }

            delete d;
            d = new Dividing(z, separator, few_fields, remove_separators);
            d->editing_fields_list(buffer, removing_katakana, field_number);
            *combo << d->unite(buffer, LISTED) << extra;
        }

    } else {
        for(;;) {
            p = fgets(buffer, buffer_size, file);
            if(p == NULL) {
                break;
            }

            delete d;
            d = new Dividing(z, separator, field_count, remove_separators);
            d->editing_fields_list(buffer, removing_katakana, field_number);
            *combo << d->unite(buffer, LISTED) << extra;
        }

    }

    new_file << combo->str();
    fclose(file);
    delete d; delete kana; delete combo;
    return exit_code;
}

int renameBinaryFile(const char *name, bool no_removing) {
    char c[BLOCK_SIZE];
    int exit_code = 0;
    int size;
    FILE *temporary = fopen(TEMPORARY_BINARY, "rb");
    FILE *recent = fopen(name, "wb");
    if(recent == NULL) {
        recent = fopen(HANZEN_TEXT, "wb");
    }

    if(temporary == NULL) {
        exit_code = FILE_ERROR_CODE;
    } else {
        do {
            size = fread(c, 1, BLOCK_SIZE, temporary);
            fwrite(c, size, 1, recent);
        } while(size == BLOCK_SIZE);

        fclose(recent);
        fclose(temporary);

        if(! no_removing) {
            remove(TEMPORARY_BINARY);
        }

    }

    return exit_code;
}

void errorHandler(const char *p, char *q) {
    using namespace std;

    ofstream errorFile(ERROR_TEXT);
    errorFile << p << GAP << q;
}

void errorHandler(const char *p, const char *q) {
    using namespace std;

    ofstream errorFile(ERROR_TEXT);
    errorFile << p << GAP << q;
}
