#include "hanzen.h"

bool binary_file_in_current_directory(char *buffer, int buffer_size) {
    using namespace std;

    bool b = true;
    ifstream in(HANKAKU_ZENKAKU_BIN); // current directory
    if( in.fail() ) {
        b = false;
        memset(buffer, 0, buffer_size);
    } else {
        strcpy(buffer, HANKAKU_ZENKAKU_BIN);
    }

    return b;
}

bool binary_file_in_installed_directory(char *buffer, int buffer_size) {
    using namespace std;

    bool b = true;
    ifstream in(C_HANKAKU_ZENKAKU_BIN); // directory where hanzen was installed.
    if( in.fail() ) {
        b = false;
        memset(buffer, 0, buffer_size);
    } else {
        strcpy(buffer, C_HANKAKU_ZENKAKU_BIN);
    }

    return b;
}

bool binary_file_in_path_directory(char *buffer, int buffer_size) {
    using namespace std;

    bool b = true;
    CommandPath *c = new CommandPath();
    string included = c->pathName(HANKAKU_ZENKAKU_BIN); // directory included in Path

    if( included.length() == 0 ) {
        b = false;
        memset(buffer, 0, buffer_size);
    } else {
        strcpy(buffer, included.c_str() );
    }

    delete c;
    return b;
}

bool SparseKana::setting(bool no_searching, bool *whichTable) {
    using namespace std;

    char buffer[BLOCK_SIZE];
    char c[HANKAKU_ZENKAKU] = {0, 0, 0};
    int buffer_size = BLOCK_SIZE;
    int index;
    OPEN_CLOSE_FUNCTION *openClose = new OPEN_CLOSE_FUNCTION[SEARCH_COUNT];
    unsigned char uc;

    openClose[0] = binary_file_in_installed_directory;
    openClose[1] = binary_file_in_current_directory;
    openClose[2] = binary_file_in_path_directory;

    for(int i = 0; i < SEARCH_COUNT; ++i) {
        if( openClose[i](buffer, buffer_size) ) {
            if(*whichTable) {
                cout << buffer << " is primarily accessible to hanzen." << endl;
                *whichTable = false;
            }

            break;
        } else {
            switch(i) {
            case 0:
                if(no_searching) {
                    cerr << "openClose[" << i << "]() error: " << HANKAKU_ZENKAKU_BIN << endl;
                    delete[] openClose;
                    return false;
                }

                break;
            case 2:
                cerr << "openClose[" << i << "]() error: " << HANKAKU_ZENKAKU_BIN << endl;
                delete[] openClose;
                return false;
            default: // case 1:
                break;
            }

        }

    }

    ifstream binaryFile(buffer);
    for(int i = 1; i <= RECORD_COUNT; ++i) {
        binaryFile.read(c, HANKAKU_ZENKAKU);

        if(i < FIRST_DAKUTEN) {
            uc = c[0];
            index = (int)uc;
            populated(index, 0, c[KATAKANA_SIZE]);
            populated(index, 1, c[ KATAKANA_SIZE + 1 ]);
        } else if (i >= FIRST_HANDAKUTEN) {
            populated( moved(c[0], HANDAKUTEN), 0, c[KATAKANA_SIZE]);
            populated( moved(c[0], HANDAKUTEN), 1, c[ KATAKANA_SIZE + 1 ]);
        } else {
            populated( moved(c[0], DAKUTEN), 0, c[KATAKANA_SIZE]);
            populated( moved(c[0], DAKUTEN), 1, c[ KATAKANA_SIZE + 1 ]);
        }

    }

    delete[] openClose;
    return true;
}

int SparseKana::moved(unsigned char c, bool handakuten) {
    int offset = handakuten ? HANDAKUTEN_OFFSET : DAKUTEN_OFFSET;
    int index = (int)c + offset;
    return index;
}

unsigned char SparseKana::dakuten(int index, int place) {
    int k = ( index + DAKUTEN_OFFSET) * ZENKAKU_SIZE_1 + place;
    unsigned char r = (unsigned char)sparse[k];
    return r;
}

unsigned char SparseKana::handakuten(int index, int place) {
    int k = ( index + HANDAKUTEN_OFFSET) * ZENKAKU_SIZE_1 + place;
    unsigned char r = (unsigned char)sparse[k];
    return r;
}

unsigned char SparseKana::working(int index, int place) {
    int k = index * ZENKAKU_SIZE_1 + place;
    unsigned char r = (unsigned char)sparse[k];
    return r;
}

SparseKana::SparseKana(int b): DAKUTEN(false), HANDAKUTEN(true) {
    bufferSize = b;
    buffer = new char[bufferSize];
    combo = new char[ bufferSize * ZENKAKU_SIZE ];

    size = 0x02E0 * ZENKAKU_SIZE_1;
    sparse = new char[size];
}

void SparseKana::clear() {
    for(int i = 0; i < size; ++i) {
        sparse[i] = 0;
    }

}

void SparseKana::populated(int index, int place, unsigned char c) {
    int k = index * ZENKAKU_SIZE_1 + place;
    sparse[k] = (char)c;
}
