#include "rufeol.h"

namespace {
    char GAP[] = " ";
}

bool RemoveFromLine::insertLF() {
    using namespace std;

    bool full = false;
    char c, h;
    int count = 0;
    int k = 0;
    ofstream temporaryFile(TEMPORARY_BINARY, ios::out | ios:: binary | ios::trunc );
    ifstream binaryFile( fileName.c_str(), ios::in | ios::binary);
    if( binaryFile.fail() ) {
        cerr << "ifstream binaryFile() error: " << fileName << endl;
        return false;
    }

// ring buffer
    ring_count = unnecessary + LF_SIZE;
    prepareRing();

    binaryFile.read(buffer, bufferSize);
    for(int m = 0; m < bufferSize; ++m) {
        c = buffer[m];
        if(c == 0x0A) {
            count += 1;
            if(count >= fullLevel) {
                full = true;
            }

        }

        ring.pop_front();
        ring.push_back(c);

        if(full) {
            combo[k++] = 0x0A;
            prepareRing();
            count = 0;
            full = false;
        } else {
            if( ringBeginsWithBell(&h) ) {
                continue; // skipping 07h
            } else {
                combo[k++] = h;
            }

        }

    } // for(int m = 0; m < bufferSize; ++m)

    temporaryFile.write(combo, k);
    return true;
}

bool RemoveFromLine::insertCRLF() {
    using namespace std;

    bool full = false;
    char c, h;
    int count = 0;
    int k = 0;
    ofstream temporaryFile(TEMPORARY_BINARY, ios::out | ios:: binary | ios::trunc );
    ifstream binaryFile( fileName.c_str(), ios::in | ios::binary);
    if( binaryFile.fail() ) {
        cerr << "ifstream binaryFile() error: " << fileName << endl;
        return false;
    }

// ring buffer
    ring_count = unnecessary + LF_SIZE;
    prepareRing();

    binaryFile.read(buffer, bufferSize);
    for(int m = 0; m < bufferSize; ++m) {
        c = buffer[m];
        if(c == 0x0A) {
            count += 1;
            if(count >= fullLevel) {
                full = true;
            }

        }

        ring.pop_front();
        ring.push_back(c);

        if(full) {
            combo[k++] = 0x0D;
            combo[k++] = 0x0A;
            prepareRing();
            count = 0;
            full = false;
        } else {
            if( ringBeginsWithBell(&h) ) {
                continue; // skipping 07h
            } else {
                combo[k++] = h;
            }

        }

    } // for(int m = 0; m < bufferSize; ++m)

    temporaryFile.write(combo, k);
    return true;
}

bool RemoveFromLine::ringBeginsWithBell(char *p) {
    bool b = false;
    char c;
    RING::iterator i = ring.begin();
    c = *i;
    if(c == BELL) {
        b = true;
    }

    *p = c;
    return b;
}

int renameBinaryFile(const char *name, bool no_removing) {
    char c[BLOCK_SIZE];
    int exit_code = 0;
    int size;
    FILE *temporary = fopen(TEMPORARY_BINARY, "rb");
    FILE *recent = fopen(name, "wb");
    if(recent == NULL) {
        recent = fopen(RUFEOL_TEXT, "wb");
    }

    if(temporary == NULL) {
        exit_code = FILE_ERROR_CODE;
    } else {
        do {
            size = fread(c, 1, BLOCK_SIZE, temporary);
            fwrite(c, size, 1, recent);
        } while(size == BLOCK_SIZE);

        fclose(recent);
        fclose(temporary);

        if(! no_removing) {
            remove(TEMPORARY_BINARY);
        }

    }

    return exit_code;
}

void errorHandler(const char *p, const char *q) {
    using namespace std;

    ofstream errorFile(ERROR_TEXT);
    errorFile << p << GAP << q;
}

void errorHandler(const char *p, std::string & q) {
    using namespace std;

    ofstream errorFile(ERROR_TEXT);
    errorFile << p << GAP << q;
}

void RemoveFromLine::prepareRing() {
    using namespace std;

    ring.clear();
    for(int i = 0; i < ring_count; ++i) {
        ring.push_back(BELL);
    }

}
