#ifndef LINE_H
#define LINE_H

#define END_OF_FILE 2
#define FILE_ERROR -1L
#include <fstream>
#include <iostream>
#include <stdio.h>
#include <string>

class Line {
    const int ERROR_CODE;
    const int LINE_COUNT_ERROR;
    const int NOT_FOUND;
    const int SEPARATOR_SIZE;
    const std::string EMPTY;

    bool carriageReturn, measure;
    char *buffer;
    int bufferSize, lineCount, lineNumber, maximum, maximumLineLength, newLineLength, point;
    std::string fileName;

public:
    int checkLineCount(bool stop) {
        int line_count = LINE_COUNT_ERROR;

        if(lineCount >= 0) {
            line_count = lineCount;
        } else {
            if(stop) {
                exit(1);
            }

        }

        return line_count;
    }

    int maximumLineSize() const {
        return maximumLineLength;
    }

    long fileSize() {
        long size = FILE_ERROR;
        FILE *p = fopen( fileName.c_str(), "rb");
        if(p == NULL) {
            std::cerr << "fopen() error-1: " << fileName << std::endl;
            return size;
        }

        fseek(p, 0, END_OF_FILE);
        size = ftell(p);
        if(size < 0L) {
            std::cerr << "ftell() error: " << size << std::endl;
            return size;
        }

        fclose(p);
        return size;
    }

    Line(std::string & f): EMPTY(""), ERROR_CODE(1), LINE_COUNT_ERROR(-1), NOT_FOUND(-1), SEPARATOR_SIZE(1)  {
        fileName = f;
        lineCount = LINE_COUNT_ERROR;
        measure = false;
        point = 1;
        bufferSize = (int)fileSize();
        if(bufferSize >= 0) {
            buffer = new char[bufferSize];

            FILE *p = fopen( fileName.c_str(), "rb");
            fread(buffer, bufferSize, 1, p);
            maximum = maximumSpan(); // carriageReturn, lineCount, lineNumber, newLineLength
            maximumLineLength = maximum - newLineLength;
            fclose(p);
        }

    }

    Line(std::string & f, int t, bool m): EMPTY(""), LINE_COUNT_ERROR(-1), ERROR_CODE(1), NOT_FOUND(-1), SEPARATOR_SIZE(1)  {
        fileName = f;
        lineCount = LINE_COUNT_ERROR;
        measure = m;
        point = t;
        bufferSize = (int)fileSize();
        if(bufferSize >= 0) {
            buffer = new char[bufferSize];

            FILE *p = fopen( fileName.c_str(), "rb");
            fread(buffer, bufferSize, 1, p);
            maximum = maximumSpan(); // carriageReturn, lineCount, lineNumber, newLineLength
            maximumLineLength = maximum - newLineLength;
            fclose(p);
        }

    }

    ~Line() {
        delete[] buffer;
    }

    bool reading();
    bool reading(int separator);
    int maximumSpan();
    int maximumSpan(int *p, int separator, int size);
};

#endif // LINE_H
