#ifndef KANJI_UNION_H
#define KANJI_UNION_H

#ifdef __BORLANDC__
#pragma warn -8004
#pragma warn -8012
#endif

#define VERSION 1.16
#define BELL 0x07
#define BELL_ASCII_KATAKANA_COUNT 224
#define BLOCK_SIZE 2560
#define BUFFER_SIZE 524288 // 512 KB
#define CARRIAGE_RETURN_LINE_FEED "-CRLF"
#define C_SIZE 256 // FILE_NAME_SIZE + 1
#define C_SJIS_EUC_BIN "C:\\sjisEUCjpCxx\\MS932~EUC-JP.bin"
#define DO_NOT_REMOVE_TEMPORARY_BINARY_FILE "-y"
#define DO_NOT_SEARCH_TABLE "-d"
#define DOUBLE_KANJI 4 // SJIS_KANJI_SIZE + EUC_KANJI_SIZE
#define ERROR_TEXT "error.txt"
#define EUC_KANJI_SIZE 2
#define EUC_KANJI_SIZE_1 3
#define FILE_ERROR_CODE -2
#define FILE_NAME_SIZE 255
#define FIRST_KANJI 159
#define IDENTICAL_TIME "-t"
#define KANA_HANDAKUTEN 0xDF
#define KANA_KUTEN 0xA1
#define KANJI_SIZE 2
#define LINE_FEED "-LF"
#define RECORD_COUNT 7882
#define SEARCH_COUNT 4
#define SKIPPING "-s"
#define SJIS_KANJI_SIZE 2
#define SJIS_EUC_BIN "MS932~EUC-JP.bin"
#define SJIS_EUC_TEXT "sjisEUC.txt"
#define TEMPORARY_BINARY_FILE "-b"
#define TEMPORARY_BINARY "binary.tmp"
#define WHICH_TABLE "-w"
#define ZERO "0"
#include "dividing.h"

typedef bool (*OPEN_CLOSE_FUNCTION)(char *buffer, int buffer_size);

typedef union KANJI KANJI;
union KANJI {
    unsigned char sequence[6];
    unsigned short code[3];
    unsigned int block;
};

class CommandPath {
    char *data;

public:
    CommandPath() {
        data = new char[BUFFER_SIZE];
        strcpy(data, getenv("Path") );
    }

    ~CommandPath() {
        delete[] data;
    }

    char *path_name(char *extra);
    std::string pathName(const char *extra);
};

class Regulate {
    const int ERROR_CODE;
    bool carriage_return, no_searching, specified_directory_has_priority, whichTable;
    char *buffer;
    std::string documentName;

public:
    Regulate(char *d, bool c, bool n, bool s, bool w): ERROR_CODE(1) {
        std::string document(d);
        documentName = document;
        carriage_return = c;
        no_searching = n;
        specified_directory_has_priority = s;
        whichTable = w;
        buffer = new char[BUFFER_SIZE];
    }

    ~Regulate() {
        delete[] buffer;
    }

    int readingToProduceEUC(char *new_name);
    int skippingControlCharactersToProduceEUC(char *new_name);
};

class Standard_input {
    const int ERROR_CODE;
    bool no_searching, specified_directory_has_priority, whichTable;
    char *buffer;

public:
    Standard_input(bool n, bool s, bool w): ERROR_CODE(1) {
        no_searching = n;
        specified_directory_has_priority = s;
        whichTable = w;
        buffer = new char[BUFFER_SIZE];
    }

    ~Standard_input() {
        delete[] buffer;
    }

    int standard_input_output();
    int standard_input_output_skipping();
};

class SparseEUC {
    char *sparse;
    int size;

public:
    SparseEUC() {
        size = 0xFC4C * EUC_KANJI_SIZE_1;
        sparse = new char[size];
    }

    ~SparseEUC() {
        delete[] sparse;
    }

    void clear() {
        memset(sparse, 0, size);
    }

    void populated(int index, int place, char c) {
        int k = index * EUC_KANJI_SIZE_1 + place;
        *(sparse + k) = c;
    }

    bool setting(bool no_searching, bool specified_directory_has_priority, bool *whichTable);
    std::string working(int index);
    unsigned char working(int index, int place);
};

bool openClose(char *file_name);
bool openClose(char *file_name, bool write);
bool openClose(std::string & fileName);
bool binary_file_in_current_directory(char *buffer, int buffer_size);
bool binary_file_in_installed_directory(char *buffer, int buffer_size);
bool binary_file_in_path_directory(char *buffer, int buffer_size);
bool binary_file_in_specified_directory(char *buffer, int buffer_size);
int inverted(char *c);
int produceBinaryFile(const char *name, bool no_removing);
int renameBinaryFile(const char *name, bool no_removing);
void acquire_parameters_from_text(const char *parameters_text, char *binary_name, char *document_name, char *new_name, bool *produce_binary_file, bool *carriage_return, bool *no_searching, bool *skip_control_characters, bool *identical_time, bool *which_table, bool *no_removing);
void errorHandler(const char *p, char *q);
void errorHandler(const char *p, const char *q);
void versionGuide(double v, bool comments);

#endif // KANJI_UNION_H
