#include "kanji_union.h"

/*
sjisEUC C++ Version 1.16
Converting the text from Shift_JIS to EUC-JP
Copyright 2019 Sogaya

gcc (GCC) 3.4.2 (mingw-special)
Borland C++ 5.5.1 for Win32
Digital Mars Compiler Version 8.42n (Digital Mars C/C++ Compiler Version 8.57)

sjisEUC assumes that the little-endian architecture underlies operating systems (Linux, Windows).
*/

char binaryName[ FILE_NAME_SIZE + 1 ];
char *binary_name = & binaryName[0];

int main(int argc, char *argv[]) {
    using namespace std;    

    const bool REMOVE_ANCIENT_FILE = false;
    const bool WITHOUT_COMMENTS = false;
    const bool WITH_COMMENTS = true;
    const char SETTING_TEXT[] = "sete.txt";
    const string ACQUIRE_TABLE_NAME_FROM_TEXT("-a");
    const string DIFFERENT_TIME("-t-");
    const string DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE("-w-");
    const string EMPTY("");
    const string NO_SKIPPING("-s-");
    const string SEARCH_TABLE("-d-");
    const string VERSION_WITH_COMMENTS("-VC");
    const string VERSION_WITHOUT_COMMENTS("-V");

    bool carriage_return = false;
    bool directory_has_priority = false;
    bool identical_time = false;
    bool no_removing = false;
    bool no_searching = false;
    bool produce_binary_file = false;
    bool skip_control_characters = false;
    bool file_to_be_read = true;
    bool which_table = false;
    char sjis_text[ FILE_NAME_SIZE + 1 ];
    char euc_text[ FILE_NAME_SIZE + 1 ];
    int file_name_size;
    int exit_code = 0;
    string *p = new string("");
    string fileName = EMPTY;
    string newName(SJIS_EUC_TEXT);
    string argument;

    for(int i = 1; i < argc; ++i) {
        delete p;
        p = new string(argv[i]);
        argument = *p;

        if(ACQUIRE_TABLE_NAME_FROM_TEXT == argument) {
            directory_has_priority = true;
        } else if(CARRIAGE_RETURN_LINE_FEED == argument) {
            carriage_return = true;
        } else if(DIFFERENT_TIME == argument) {
            identical_time = false;
        } else if(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE == argument) {
            no_removing = true;
        } else if(DO_NOT_SEARCH_TABLE == argument) {
            no_searching = true;
        } else if(DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE == argument) {
            which_table = false;
        } else if(IDENTICAL_TIME == argument) {
            identical_time = true;
        } else if(LINE_FEED == argument) {
            carriage_return = false;
        } else if(NO_SKIPPING == argument) {
            skip_control_characters = false;
        } else if(SEARCH_TABLE == argument) {
            no_searching = false;
        } else if(SKIPPING == argument) {
            skip_control_characters = true;
        } else if(TEMPORARY_BINARY_FILE == argument) {
            produce_binary_file = true;
        } else if(VERSION_WITH_COMMENTS == argument) {
            delete p;
            versionGuide(VERSION, WITH_COMMENTS);
        } else if(VERSION_WITHOUT_COMMENTS == argument) {
            delete p;
            versionGuide(VERSION, WITHOUT_COMMENTS);
        } else if(WHICH_TABLE == argument) {
            which_table = true;
        } else {
            if(file_to_be_read) {
                fileName = argument;
                file_to_be_read = false;
            } else {
                newName = argument;
            }

        }

    }

    file_name_size = fileName.length();
    if(file_name_size == 0) {
        Standard_input *s = new Standard_input(no_searching, directory_has_priority, which_table);

        if(skip_control_characters) {
            exit_code = s->standard_input_output_skipping();
        } else {
            exit_code = s->standard_input_output();
        }

        delete s;
    } else {
        sjis_text[FILE_NAME_SIZE] = '\0';
        strncpy(sjis_text, fileName.c_str(), FILE_NAME_SIZE);

        if(identical_time) {
            produceBinaryFile(sjis_text, REMOVE_ANCIENT_FILE);
            renameBinaryFile(sjis_text, no_removing);
        }

        Regulate *r = new Regulate(sjis_text, carriage_return, no_searching, directory_has_priority, which_table);

        euc_text[FILE_NAME_SIZE] = '\0';
        strncpy(euc_text, newName.c_str(), FILE_NAME_SIZE);

        if(produce_binary_file) {
            if(skip_control_characters) {
                exit_code = r->skippingControlCharactersToProduceEUC(TEMPORARY_BINARY);
            } else {
                exit_code = r->readingToProduceEUC(TEMPORARY_BINARY);
            }

            renameBinaryFile(euc_text, no_removing);
        } else {
            if(skip_control_characters) {
                exit_code = r->skippingControlCharactersToProduceEUC(euc_text);
            } else {
                exit_code = r->readingToProduceEUC(euc_text);
            }

        }


        delete r;
    }

    delete p;
    return exit_code;
}

/*
1:  binary_name                       MS932~EUC-JP.bin
2:  document_name                     KeyWords.csv
3:  new_name                          sjisEUC.txt
4:  TEMPORARY_BINARY_FILE             -b
5:  CARRIAGE_RETURN_LINE_FEED         -CRLF
6:  DO_NOT_SEARCH_TABLE               -d
7:  SKIPPING                          -s
8:  IDENTICAL_TIME                    -t
9:  WHICH_TABLE                       -w
10: DO_NOT_REMOVE_TEMPORARY_BINARY    -y
*/

void acquire_parameters_from_text(const char *parameters_text, char *binary_name, char *document_name, char *new_name, bool *produce_binary_file, bool *carriage_return, bool *no_searching, bool *skip_control_characters, bool *identical_time, bool *which_table, bool *no_removing) {
    char *p;
    char c[C_SIZE];
    FILE *setting = fopen(parameters_text, "rt");
    int k;
    int line_number = 0;

    if(setting != NULL) {
        for(;;) {
            p = fgets(c, C_SIZE, setting);
            if(p == NULL) {
                break;
            } else {
                k = strcspn(c, "\n',|\t");
                c[k] = '\0';
            }

            line_number += 1;
            switch(line_number) {
            case 1:
                strcpy(binary_name, c); // MS932~EUC-JP.bin
                break;
            case 2:
                strcpy(document_name, c); // KeyWords.csv
                break;
            case 3:
                strcpy(new_name, c); // sjisEUC.txt
                break;
            case 4:
                if(! strcmp(TEMPORARY_BINARY_FILE, c) ) { // -b
                    *produce_binary_file = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *produce_binary_file = false;
                } else {
                    *produce_binary_file = true;
                }

                break;
            case 5:
                if(! strcmp(CARRIAGE_RETURN_LINE_FEED, c) ) { // -CRLF
                    *carriage_return = true;
                } else if(! strcmp(LINE_FEED, c) ) { // -LF
                    *carriage_return = false;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *carriage_return = false;
                } else {
                    *carriage_return = true;
                }

                break;
            case 6:
                if(! strcmp(DO_NOT_SEARCH_TABLE, c) ) { // -d
                    *no_searching = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *no_searching = false;
                } else {
                    *no_searching = true;
                }

                break;
            case 7:
                if(! strcmp(SKIPPING, c) ) { // -s
                    *skip_control_characters = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *skip_control_characters = false;
                } else {
                    *skip_control_characters = true;
                }

                break;
            case 8:
                if(! strcmp(IDENTICAL_TIME, c) ) { // -t
                    *identical_time = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *identical_time = false;
                } else {
                    *identical_time = true;
                }

                break;
            case 9:
                if(! strcmp(WHICH_TABLE, c) ) { // -w
                    *which_table = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *which_table = false;
                } else {
                    *which_table = true;
                }

                break;
            case 10:
                if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, c) ) { // -y
                    *no_removing = true;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *no_removing = false;
                } else {
                    *no_removing = true;
                }

                break;
            default:
                break;
            } // switch(line_number)

        } // for(;;)

        fclose(setting);
    } // if(setting != NULL)

}
